#include "mainwindow.h"

#include <QShortcut>
#include <QDialog>
#include <QDialogButtonBox>
#include <QFormLayout>
#include <QLineEdit>
#include <QFile>
#include <QTextStream>
#include <QString>
#include <QCoreApplication>
#include <QSqlDatabase>
#include <QMouseEvent>
#include <iostream>
#include <QSqlQuery>
#include <QSet>
#include <QSqlError>
#include <QDebug>
#include <fstream>
#include <QMessageBox>
#include <QSqlRecord>
#include <QSqlTableModel>
#include <QList>
#include <QSqlDatabase>
#include <iomanip>
#include <windows.h>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    menu = new QMenu("&Menu");
    menuBar = new QMenuBar(this);
    menu->addAction("Exit", this, SLOT(on_actionExit_triggered()), Qt::CTRL+Qt::Key_Q);
    menu->addAction("Connect", this, SLOT(on_actionConnect_triggered()), Qt::CTRL+Qt::Key_W);

    menuBar->addMenu(menu);
    setMenuBar(menuBar);
    mainWidget = new QWidget();

    lineEdit = new QTextEdit(this);
    textEdit_log = new QTextEdit(this);
    label = new QLabel(this);
    listWidget = new QListWidget(this);
    pushButton = new QPushButton(this);
    pushButton_2 = new QPushButton(this);
    tableView = new QTableView(this);

    pushButton->setText("Ok");
    pushButton_2->setText("Clear");

    label->setText("Список таблиц");
    QString StyleSheet =   "QLabel { color: black; border: none; font: 14pt 'arial'; font-weight: 900; }";
    label->setStyleSheet(StyleSheet);
    label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);

    tableView->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    textEdit_log->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Minimum);
    lineEdit->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    label->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    listWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
    pushButton_2->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);

    QString StyleSheet_2 =   "QTextEdit { color: black; border: none; font: 10pt 'consolas'; }";
    QString StyleSheet_3 =   "QTextEdit { color: black; border: none; font: 12pt 'consolas'; font-weight: 600; }";
    lineEdit->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    lineEdit->setStyleSheet(StyleSheet_2);
    lineEdit->setPlaceholderText("Enter the command");
    lineEdit->setDisabled(true);
    textEdit_log->setReadOnly(true);
    textEdit_log->setStyleSheet(StyleSheet_3);
    textEdit_log->setPlaceholderText("Primarily connect to Data Base");


    QGridLayout *layout = new QGridLayout(this);
    layout->setSpacing(11);

    layout->addWidget(label, 0, 0, 1, 2);
    layout->addWidget(listWidget, 1, 0, 5, 2);
    layout->addWidget(tableView, 0, 2, 5, 3);
    layout->addWidget(textEdit_log, 5, 2, 1, 3);
    layout->addWidget(lineEdit, 6, 0, 3, 4);
    layout->addWidget(pushButton, 6, 4, 1, 1);
    layout->addWidget(pushButton_2, 8, 4, 1, 1);

    mainWidget->setLayout(layout);
    setCentralWidget(mainWidget);
    centralWidget()->layout()->setContentsMargins(11, 11, 11, 11);

    connect(pushButton, SIGNAL(clicked()), this, SLOT(ok_clicked()));
    connect(pushButton_2, SIGNAL(clicked()), this, SLOT(clear_clicked()));
    connect(listWidget, &QListWidget::itemClicked, this, [this](QListWidgetItem* item)
    {
        DBUpdate(item->text());
    });

   // QShortcut *shortcut = new QShortcut(Qt::SHIFT + Qt::Key_Return, this);
   // QObject::connect(shortcut, SIGNAL(activated()), this, SLOT(ok_clicked()));
    pushButton->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Return));
}

void MainWindow::ok_clicked() {
    QString temp = lineEdit->toPlainText();
    QString command;
    for (int i = 0; i < temp.size(); i++) {
        if (i == 0) {
            command.append(temp[i]);
        }
        else {
            if (!(temp[i-1] == "\n" && temp[i] == "\n")) {
                command.append(temp[i]);
            }
        }

    }
    QSqlQuery query;

    bool cmd = query.exec(command);
    std::string str = command.toStdString();

    if (command != nullptr) {
        if (!cmd) {
            QMessageBox::warning(this, "Warning", query.lastError().text());
        }
        else {
            textEdit_log->append(command);
            textEdit_log->append("");

            if (str.substr(0, 6) == "SELECT" || str.substr(0, 4) == "WITH") {

                QSqlQueryModel *model_q = new QSqlQueryModel(tableView);
                model_q->setQuery(command);
                tableView->setModel(model_q);
            }
        }

        if (!(str.substr(0, 6) == "SELECT" || str.substr(0, 4) == "WITH")){

            QStringList table_list = myDB.tables();
            bool a = true;
            for(int i = 0; i < table_list.size(); i++){
                if (command.indexOf(table_list[i]) != -1) {
                    DBUpdate(table_list[i]);
                    a = false;
                }
            }
            if (a) DBUpdate("");
        }
    }
}



void MainWindow::clear_clicked() {
    lineEdit->clear();
}

MainWindow::~MainWindow() {}


void MainWindow::on_actionExit_triggered()
{
      QMessageBox::StandardButton ret;
      ret = QMessageBox::question( this,  QApplication::applicationName(), tr(" Do you want to close programm? "),
                                   QMessageBox::Yes | QMessageBox::No );
      if (ret == QMessageBox::Yes) {

          ret = QMessageBox::question( this,  QApplication::applicationName(), tr("Do you want to save your command\n"
                                                                                      "history? "),
                                           QMessageBox::Yes | QMessageBox::No );
          if (ret == QMessageBox::Yes) {
              QFile file_1("history.txt");
              QStringList list;
              QString str;
              QSet<QString> set;

              if(file_1.open(QIODevice::WriteOnly | QIODevice::Text)) {
                      QTextStream out(&file_1);

                      out << textEdit_log->toPlainText();
                      file_1.close();
              }

              if(file_1.open(QIODevice::ReadOnly)) {
                  QTextStream in(&file_1);

                  file_1.seek(SEEK_SET);
                  while (!file_1.atEnd()) {
                      str = in.readLine();
                      list.append(str);
                  }
                  file_1.seek(SEEK_SET);
                  while (!file_1.atEnd()) {
                      str = in.readLine();
                      for (const auto& i: list) { set.insert(i); }
                  }

                  file_1.remove();
              }

              QFile file("history.txt");
              if(file.open(QIODevice::WriteOnly | QIODevice::Text)) {
                      QTextStream out(&file);

                      for (const auto& i: set) { out << i; }
              }

              CloseDB();
              QApplication::quit();

          }

          if (ret == QMessageBox::No) {
              QFile file("history.txt");
              file.remove();
              CloseDB();
              QApplication::quit();
          }
      }
}

void MainWindow::showEvent(QShowEvent *event) {
    QMessageBox::StandardButton ret;

    ret = QMessageBox::question( this,  QApplication::applicationName(), tr("Do you want to upload command\n"
                                                                            "history? "),
                                 QMessageBox::Yes | QMessageBox::No );


    if (ret == QMessageBox::Yes) {
        std::fstream file("history.txt");
        if (file.peek() != EOF) {
            first = true;
        } else  QMessageBox::warning(this, "Warning", "File wasn't founded");
    }
    else { event->ignore(); }

}

void MainWindow::closeEvent(QCloseEvent *event) {

    QMessageBox::StandardButton ret;
    ret = QMessageBox::question( this,  QApplication::applicationName(), tr("Do you want to close programm? "),
                                 QMessageBox::Yes | QMessageBox::No );

    if (ret == QMessageBox::Yes) {
        ret = QMessageBox::question( this,  QApplication::applicationName(), tr("Do you want to save your command\n"
                                                                                "history? "),
                                     QMessageBox::Yes | QMessageBox::No );
        if (ret == QMessageBox::Yes) {
            QFile file_1("history.txt");
            QStringList list;
            QString str;
            QSet<QString> set;

            if(file_1.open(QIODevice::WriteOnly | QIODevice::Text)) {
                    QTextStream out(&file_1);

                    out << textEdit_log->toPlainText();
                    file_1.close();
            }

            if(file_1.open(QIODevice::ReadOnly)) {
                QTextStream in(&file_1);

                file_1.seek(SEEK_SET);
                while (!file_1.atEnd()) {
                    str = in.readLine();
                    list.append(str);
                }
                file_1.seek(SEEK_SET);
                while (!file_1.atEnd()) {
                    str = in.readLine();
                    for (const auto& i: list) { set.insert(i); }
                }
                file_1.remove();
            }

            QFile file("history.txt");
            if(file.open(QIODevice::WriteOnly | QIODevice::Text)) {
                    QTextStream out(&file);

                    for (const auto& i: set) { out << i; }
            }

        }

        if (ret == QMessageBox::No) {
            QFile file("history.txt");
            file.remove();
            CloseDB();
            QApplication::quit();
        }
    }

    if (ret == QMessageBox::No)
        event->ignore();

}

void MainWindow::checkFlags(bool& first, bool& sec) {
    QFile file("history.txt");
    QString str;
    QTextStream in(&file);

    if (first == true && sec == true) {
        if (file.open(QIODevice::ReadOnly) ) {
            textEdit_log->append(in.readAll());
            textEdit_log->append("");
            file.close();
        }
    }
}

void MainWindow::on_actionConnect_triggered()
{
    QDialog dlg(this);
    dlg.setWindowTitle(tr("Connect configuration"));
    dlg.setWindowIcon(QIcon("D:\\Programms\\Qt Labs\\manager_db\\Manager_DB_\\logo_connection.png"));
    dlg.isModal(); //делаем приоритетным окном
    dlg.setWindowFlags (dlg.windowFlags() & ~Qt::WindowContextHelpButtonHint); //убираем знак вопроса
    QLineEdit *SqlDatabase = new QLineEdit(&dlg);
    QLineEdit *HostName = new QLineEdit(&dlg);
    QLineEdit *DatabaseName = new QLineEdit(&dlg);
    QLineEdit *UserName = new QLineEdit(&dlg);
    QLineEdit *Password = new QLineEdit(&dlg);
    QLineEdit *Port = new QLineEdit(&dlg);

    QDialogButtonBox *btn_box = new QDialogButtonBox(&dlg);
    btn_box->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(btn_box, &QDialogButtonBox::accepted, &dlg, &QDialog::accept);
    connect(btn_box, &QDialogButtonBox::rejected, &dlg, &QDialog::reject);

    QFormLayout *layout = new QFormLayout();
    layout->addRow(tr("SqlDatabase:"), SqlDatabase);
    layout->addRow(tr("HostName:"), HostName);
    layout->addRow(tr("DatabaseName:"), DatabaseName);
    layout->addRow(tr("UserName:"), UserName);
    layout->addRow(tr("Password:"), Password);
    layout->addRow(tr("Port:"), Port);
    layout->addWidget(btn_box);

    //Пошла работа с файлами
    QFile file("config.txt");

    if (file.open(QIODevice::ReadOnly)) {
        QTextStream in(&file);
        SqlDatabase->setText(config.SqlDatabase = in.readLine());
        HostName->setText(config.HostName = in.readLine());
        DatabaseName->setText(config.DatabaseName = in.readLine());
        UserName->setText(config.UserName = in.readLine());
        Password->setText(config.Password = in.readLine());
        Port->setText(config.Port = in.readLine());
        file.close();
    }

    dlg.setLayout(layout);

    if(dlg.exec() == QDialog::Accepted) {

        if(file.open(QIODevice::WriteOnly | QIODevice::Text))
            {
                QTextStream out(&file);
                out << SqlDatabase->text() << "\n";
                config.SqlDatabase = SqlDatabase->text();
                out << HostName->text() << "\n";
                config.HostName = HostName->text();
                out << DatabaseName->text() << "\n";
                config.DatabaseName = DatabaseName->text();
                out << UserName->text() << "\n";
                config.UserName = UserName->text();
                out << Password->text() << "\n";
                config.Password = Password->text();
                out << Port->text() << "\n";
                config.Port = Port->text();
                file.close();
        }
        ConnectToServer();
    }
}

void MainWindow::ConnectToServer() {
    myDB = QSqlDatabase::addDatabase(config.SqlDatabase);
    myDB.setHostName(config.HostName);
    myDB.setDatabaseName(config.DatabaseName);
    myDB.setUserName(config.UserName);
    myDB.setPassword(config.Password);
    myDB.setPort(config.Port.toInt());

    if (!myDB.open()) {
        QMessageBox::warning(this, "Warning", myDB.lastError().text());
    }
    else {
        sec = true;
        checkFlags(first, sec);
        lineEdit->setDisabled(false);
    }
    DBUpdate("");
}


void MainWindow::DBUpdate(const QString table_name) {

    QSqlTableModel *model = new QSqlTableModel(this);

    QStringList table_list = myDB.tables();
    listWidget->clear();
    listWidget->addItems(table_list);

    QStringList headers;

    if (table_name.size()) {
         model->setTable(table_name);
    }
    else if (table_list.size()) {
        model->setTable(table_list[0]);
    }

    for (int i = 0; i < model->columnCount(); i++)
    {
        headers.append(model->headerData(i, Qt::Horizontal).toString());
    }

    for (int i = 0, j = 0; i < model->columnCount(); i++, j++)
    {
        model->setHeaderData(i, Qt::Horizontal, headers[j]);
    }


    tableView->setModel(model);
    model->select();
}

void MainWindow::CloseDB() {

    myDB.close();
}

